searchData={"items":[{"type":"module","title":"A2S","doc":"A collection of pure functions for forming A2S challenges, requests, and parsing responses.","ref":"A2S.html"},{"type":"function","title":"A2S.challenge_request/1","doc":"Returns the challenge request packet for the given query type.","ref":"A2S.html#challenge_request/1"},{"type":"function","title":"A2S.parse_challenge/1","doc":"Parses a challenge response packet. Some game servers don't implement the challenge flow and will\nimmediately respond with the requested data. In that case `:immediate` will be returned with the data.\n\nIf the server returns data immediately, and that data is multipacket, `:multipacket` will be returned.","ref":"A2S.html#parse_challenge/1"},{"type":"function","title":"A2S.parse_challenge!/1","doc":"","ref":"A2S.html#parse_challenge!/1"},{"type":"function","title":"A2S.parse_multipacket_response/1","doc":"","ref":"A2S.html#parse_multipacket_response/1"},{"type":"function","title":"A2S.parse_multipacket_response!/1","doc":"","ref":"A2S.html#parse_multipacket_response!/1"},{"type":"function","title":"A2S.parse_response/1","doc":"","ref":"A2S.html#parse_response/1"},{"type":"function","title":"A2S.parse_response!/1","doc":"","ref":"A2S.html#parse_response!/1"},{"type":"function","title":"A2S.sign_challenge/2","doc":"","ref":"A2S.html#sign_challenge/2"},{"type":"module","title":"A2S.Client","doc":"An easy to use client that performs the packet assembly necessary to execute A2S queries.\n\nThis client handles requests concurrently and should be suitable for most general uses.\n\nNote: must supply a `:name`.","ref":"A2S.Client.html"},{"type":"function","title":"A2S.Client.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"A2S.Client.html#child_spec/1"},{"type":"function","title":"A2S.Client.query/3","doc":"Query a game server running at `address` for the data specified by `query`.\n\nAdditional options are available as a keyword list:\n* `:name` - Alias of the top-level supervisor. Defaults to `A2S.Client`.\n\n* `:timeout` - Absolute timeout for the request to complete. Defaults to `5000` or 5 seconds.","ref":"A2S.Client.html#query/3"},{"type":"function","title":"A2S.Client.start_link/1","doc":"The following configuration options are available:\n\n* `:name` - Alias of the top-level supervisor. Can be provided if you intend to run multiple client instances. Defaults to `A2S_Singleton`.\n\n* `:port` - UDP socket port used for querying game servers. Defaults to `20850`.\n\n* `:idle_timeout` - `A2S.Client` internally allocates a `:gen_statem` for each address queried, that should exit after idling to keep from leaking resources. Defaults to `120_000` or 2 minutes.\n\n* `:recv_timeout` - Timeout the `:gen_statem`'s will wait between receiving individual packets before returning `{:error, :recv_timeout}`. Defaults to `3000` or 3 seconds.","ref":"A2S.Client.html#start_link/1"},{"type":"module","title":"A2S.Info","doc":"Struct representing an [A2S_INFO](https://developer.valvesoftware.com/wiki/Server_queries#Response_Format) response.","ref":"A2S.Info.html"},{"type":"type","title":"A2S.Info.t/0","doc":"","ref":"A2S.Info.html#t:t/0"},{"type":"module","title":"A2S.MultiPacketHeader","doc":"Struct representing a [multi-packet response header](https://developer.valvesoftware.com/wiki/Server_queries#Multi-packet_Response_Format).","ref":"A2S.MultiPacketHeader.html"},{"type":"type","title":"A2S.MultiPacketHeader.t/0","doc":"","ref":"A2S.MultiPacketHeader.html#t:t/0"},{"type":"module","title":"A2S.Player","doc":"Struct representing a player entry in an [A2S_PLAYER](https://developer.valvesoftware.com/wiki/Server_queries#Response_Format_2) response.","ref":"A2S.Player.html"},{"type":"type","title":"A2S.Player.t/0","doc":"","ref":"A2S.Player.html#t:t/0"},{"type":"module","title":"A2S.Players","doc":"Struct representing an [A2S_PLAYER](https://developer.valvesoftware.com/wiki/Server_queries#Response_Format_2) response.","ref":"A2S.Players.html"},{"type":"type","title":"A2S.Players.t/0","doc":"","ref":"A2S.Players.html#t:t/0"},{"type":"module","title":"A2S.Rules","doc":"Struct representing an [A2S_RULES](https://developer.valvesoftware.com/wiki/Server_queries#Response_Format_3) response.","ref":"A2S.Rules.html"},{"type":"type","title":"A2S.Rules.t/0","doc":"","ref":"A2S.Rules.html#t:t/0"},{"type":"exception","title":"A2S.ParseError","doc":"Struct representing an error encountered in parsing an A2S response.","ref":"A2S.ParseError.html"},{"type":"function","title":"A2S.ParseError.message/1","doc":"","ref":"A2S.ParseError.html#message/1"},{"type":"type","title":"A2S.ParseError.t/0","doc":"","ref":"A2S.ParseError.html#t:t/0"},{"type":"extras","title":"Elixir A2S","doc":"# Elixir A2S\nA library for communicating with game servers running [Valve's A2S server query protocol](https://developer.valvesoftware.com/wiki/Server_queries).","ref":"readme.html"},{"type":"extras","title":"Installation - Elixir A2S","doc":"Add `:elixir_a2s` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:elixir_a2s, \"~> 0.4.3\"}\n  ]\nend\n```\n\nDocumentation is available on [HexDocs](https://hexdocs.pm/elixir_a2s/readme.html) and may also be generated with [ExDoc](https://github.com/elixir-lang/ex_doc).","ref":"readme.html#installation"},{"type":"extras","title":"Usage - Elixir A2S","doc":"There's two general ways to use this library:\n\n### `A2S.Client`\nAn easy to use client that should cover most use cases.\n\nAdd `A2S.Client` to your app's supervision tree:\n```elixir\nchildren = [\n  {A2S.Client, []}\n]\n```\nOr start the client dynamically: \n```elixir\nA2S.Client.start_link()\n```\n\nAfterwards, querying a game server's as simple as:\n```elixir\nA2S.Client.query(:info, {{127, 0, 0, 1}, 20000}) # ipv4 address followed by the query port\n```\n\n#### Notes:\n* `A2S.Client` assumes a singleton pattern by default.\n\n* For configuring multiple instances, see `A2S.Client.start_link/1` and `A2S.Client.query/3`.\n\n### `A2S`\nThis module provides functions form requests, sign challenges, and parse responses for the A2S protocol. You can utilize this module directly in your application for tighter integration, but in turn you'll have to roll your own packet assembly. See [Using A2S Directly](pages/using-a2s-directly.md) guide for further details.","ref":"readme.html#usage"},{"type":"extras","title":"Unsupported games and features - Elixir A2S","doc":"#### [Source 2006](https://en.wikipedia.org/wiki/Source_(game_engine)#Source_2006) (aka \"Pre-Orange Box\") servers\nMost source games are running on newer versions of the engine.\n\n#### GoldSrc servers not using the standard protocol\nMany GoldSrc servers use the current standard, so this should impact few games.\n\n#### [The Ship](https://steamcharts.com/app/2400)\nUses proprietary fields only worth supporting for posterity-sake.\n\n#### [A2A_PING](https://developer.valvesoftware.com/wiki/Server_queries#A2A_PING)\nConsidered deprecated by Valve and is unsupported by almost all most games.\n\n#### [A2S_SERVERQUERY_GETCHALLENGE](https://developer.valvesoftware.com/wiki/Server_queries#A2S_SERVERQUERY_GETCHALLENGE)\nOnly used by a handful of niche games, and the normal challenge flow should work anyway.","ref":"readme.html#unsupported-games-and-features"},{"type":"extras","title":"Debugging - Elixir A2S","doc":"By default [Elixir will ignore `:gen_statem` crashes](https://elixirforum.com/t/why-does-logger-translator-ignore-gen-statem-reports/37418). To receive them, add the following to your application:\n```elixir\nLogger.add_translator({A2S.StateMachineTranslator, :translate})\n```","ref":"readme.html#debugging"},{"type":"extras","title":"Using A2S Directly","doc":"# Using `A2S` Directly\n\nIf `A2S.Client`'s process architecture isn't suitable for your needs, you can use the parsing functions in `A2S` directly. In order to do so, you'll need to understand the steps necessary to perform A2S queries.","ref":"using-a2s-directly.html"},{"type":"extras","title":"A rough visual guide - Using A2S Directly","doc":"Each colored box is some internal action taken by your app. Each arrow represents a game server reply.","ref":"using-a2s-directly.html#a-rough-visual-guide"},{"type":"extras","title":"In a perfect world this would be the flow: - Using A2S Directly","doc":"*Where the 'challenge step' is necessary to mitigate A2S queries from being used in reflection attacks.*\n![In a perfect world](assets/in-a-perfect-world.svg)","ref":"using-a2s-directly.html#in-a-perfect-world-this-would-be-the-flow"},{"type":"extras","title":"How it actually works: - Using A2S Directly","doc":"*Given the A2S protocol's used by a tons of games in various states of implementation.* ([fullscreen](assets/how-it-actually-works.svg))\n![How it actually works](assets/how-it-actually-works.svg)\n\n[`A2S.Statem`](https://github.com/Moosieus/elixir-a2s/blob/main/lib/a2s/statem.ex) may be of good reference for how to handle this in your own app.","ref":"using-a2s-directly.html#how-it-actually-works"},{"type":"extras","title":"Other notes - Using A2S Directly","doc":"- `A2S.Client` is specifically optimized for querying numerous game servers on a frequent basis.\n- Official Valve servers may not respond to A2S queries.\n- Some game servers ignore certain fields, filling them with zero values.\n- [A2S_INFO](https://steamcommunity.com/discussions/forum/14/2974028351344359625/) was updated relatively recently to the rest of the protocol to mitigate reflection attacks.\n- See Valve's [official A2S documentation](https://developer.valvesoftware.com/wiki/Server_queries) for more details on A2S.","ref":"using-a2s-directly.html#other-notes"},{"type":"extras","title":"Passive Mode Example","doc":"# Passive Mode Example","ref":"passive_example.html"},{"type":"extras","title":"Setup - Passive Mode Example","doc":"This LiveBook demonstrates the minimal viable code to query a game server using `:gen_udp` in passive mode and the `A2S` module.\n\n```elixir\nMix.install(\n  [\n    {:elixir_a2s, path: Path.join(__DIR__, \"../\"), env: :dev}\n  ],\n  lockfile: :elixir_a2s\n)\n\nparse_ip = fn addr when is_list(addr) ->\n  [rest, port] = :string.split(addr, ~c\":\")\n  {:ok, ip} = :inet.parse_ipv4_address(rest)\n  {port, []} = :string.to_integer(port)\n  {ip, port}\nend\n```","ref":"passive_example.html#setup"},{"type":"extras","title":"Parameters - Passive Mode Example","doc":"```elixir\n# target server\ngame_server = ~c\"216.52.148.47:27015\"\ngame_server = ~c\"127.0.0.1:27015\"\n# timeout\ntimeout = 3000\n# udp port (probably dont need to change this)\nsource_port = 20850\n\naddress = parse_ip.(game_server)\n```","ref":"passive_example.html#parameters"},{"type":"extras","title":"Query the server - Passive Mode Example","doc":"`try/rescue`'s used to ensure the `:gen_udp` `socket` is closed even if an exception occurs.\n\n```elixir\n{:ok, socket} = :gen_udp.open(source_port, [:binary, {:active, false}])\n\ntry do\n  :ok = :gen_udp.send(socket, address, A2S.challenge_request(:info))\n\n  {:ok, {_addr, _port, challenge}} = :gen_udp.recv(socket, 0, timeout)\n\n  case A2S.parse_challenge(challenge) do\n    {:immediate, {:info, info}} ->\n      IO.inspect(info, label: \"info\", limit: :infinity)\n\n    {:challenge, challenge} ->\n      :ok = :gen_udp.send(socket, address, A2S.sign_challenge(:info, challenge))\n      {:ok, {_addr, _port, data}} = :gen_udp.recv(socket, 0, timeout)\n      {:info, info} = A2S.parse_response(data)\n      IO.inspect(info, label: \"info\", limit: :infinity)\n  end\nrescue\n  e -> IO.puts(Exception.format(:error, e, __STACKTRACE__))\nend\n\n:gen_udp.close(socket)\n```","ref":"passive_example.html#query-the-server"}],"content_type":"text/markdown","producer":{"name":"ex_doc","version":[48,46,51,52,46,49]}}